from argparse import ArgumentTypeError
from z.template import apply_template 
from pathlib import Path
from typing import List, Union
import datetime
import re

# Format: <timestamp>--<title>__<tag1>_<tag2>.<ext>
def normalize_name(name: str, tags: str, time: Union[str, None] = None, ext: str ="md"):
    if "--" in name or "__" in name:
        raise ArgumentTypeError("Title cannot have '--' or '__'")
    if "--" not in tags and "__" not in tags:
        if "_" in tags and "," in tags:
            raise ArgumentTypeError("Tags must be seperated by ',' or '_'")
        elif "_" in tags:
            tag_sep = "_"
        else:
            tag_sep = ","
    else:
        raise ArgumentTypeError("Tags cannot have '--' or '__'")

    # timestamp
    if time:
        if not re.match("[0-9]{8}T[0-9]{6}", time):
            raise ArgumentTypeError("Time must be string in format: YYMMDDTHHmmSS")
        f = time
    else:
        f = datetime.datetime.now().strftime("%Y%m%dT%H%M%S")

    # title
    f += "--"
    f += re.sub(r" ", "-", name).strip()

    # tags
    f += "__"
    for tag in tags.split(tag_sep):
        tag = tag.strip()
        tag = re.sub(r" ", "-", tag)
        f += tag + "_"
    f = f[:-1]

    # ext
    if ext + " "[0] == ".":
        f += ext
    else:
        f += "." + ext
    return f

def create_file(name: str, silo: str, content: str = ""):
    if silo:
        Path(silo).mkdir(exist_ok=True)
        if silo[-1] != "/":
            silo += "/"
    with open(silo + name, "w") as f:
        f.write(content)
    return

def main(args):
    file = normalize_name(args.name, args.tags)
    if args.outline:
        content = apply_template(args, Path.home().joinpath(".config/z-py/templates"), args.outline)
    else:
        content = apply_template(args, Path.home().joinpath(".config/z-py/templates"), "default.md")
    create_file(file, args.silo, content=content) 

if __name__ == "__main__":
    main(None)
